# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import winrt.system
from winrt._winrt_windows_ui_xaml_resources import (
    CustomXamlResourceLoader,
)
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from winrt._winrt_windows_ui_xaml_resources import (
        CustomXamlResourceLoader_Static,
    )

__all__ = [
    "CustomXamlResourceLoader",
]

